local T, C, L, _ = unpack(select(2, ...))

----------------------------------------------------------------------------------------
--	Temporary stuff
----------------------------------------------------------------------------------------
function T.ChangeSize(frame, x, y)
	local w, h = frame:GetSize()
	frame:SetSize(w + x, h + y)
end

function T.ChangePoint(frame, x, y)
	local relF, parent, relT, xoffset, yoffset = frame:GetPoint()
	frame:SetPoint(relF, parent, relT, xoffset + x , yoffset + y)
end

local borderr, borderg, borderb, bordera = unpack(C.media.classborder_color)
local function Menu_OnEnter(self)
	self.backdrop:SetBackdropBorderColor(borderr, borderg, borderb)
end
local function Menu_OnLeave(self)
	self.backdrop:SetBackdropBorderColor(1, 1, 1, 0.2)
end
local function Menu_OnMouseUp(self)
	self.backdrop:SetBackdropColor(0, 0, 0, 0.6)
end
local function Menu_OnMouseDown(self)
	self.backdrop:SetBackdropColor(borderr, borderg, borderb, 0.25)
end

local function ReskinMenuButton(self)
	self:StripTextures()
	self:CreateBackdrop("Transparent")

	self:SetScript("OnEnter", Menu_OnEnter)
	self:SetScript("OnLeave", Menu_OnLeave)
	self:HookScript("OnMouseUp", Menu_OnMouseUp)
	self:HookScript("OnMouseDown", Menu_OnMouseDown)
end

local function GetToggleDirection(direc)
	if direc == 1 then
		return ">", "<", "RIGHT", "LEFT", -3, 0, 9, 70
	elseif direc == 2 then
		return "<", ">", "LEFT", "RIGHT", 3, 0, 9, 70
	elseif direc == 3 then
		return "∨", "∧", "BOTTOM", "TOP", 0, 3, 70, 9
	else
		return "∧", "∨", "TOP", "BOTTOM", 0, -3, 70, 9
	end
end

local function SetToggleDirection(frame)
	local str1, str2, rel1, rel2, x, y, width, height = GetToggleDirection(1)
	local parent = frame.backdrop
	local close = frame.closeButton
	local open = frame.openButton
	close:ClearAllPoints()
	close:SetPoint(rel1, parent, rel2, x, y)
	close:SetSize(width, height)
	open:ClearAllPoints()
	open:SetPoint(rel1, parent, rel1, -x, -y)
	open:SetSize(width, height)
	close:SetScale(1)
	close:SetAlpha(1)
	open:SetScale(1)
	open:SetAlpha(1)
	close.text:SetText(str1)
	open.text:SetText(str2)
end

local function CreateToggleButton(parent)
	local bu = CreateFrame("Button", nil, parent)
	bu:SetSize(10, 80)
	bu.text = bu:CreateFontString(nil, "OVERLAY")
	bu.text:SetFont(C.font.stylization_font, C.font.stylization_font_size, C.font.stylization_font_style)
	bu.text:SetWordWrap(false)
	bu.text:SetTextColor(unpack(C.media.classborder_color))
	bu.text:SetPoint("CENTER", -1, 0)
	ReskinMenuButton(bu)

	return bu
end

function T.CreateToggle(frame)
	local close = CreateToggleButton(frame)
	frame.closeButton = close

	local open = CreateToggleButton(UIParent)
	open:Hide()
	frame.openButton = open

	open:SetScript("OnClick", function()
		open:Hide()
	end)
	close:SetScript("OnClick", function()
		open:Show()
	end)

	SetToggleDirection(frame)

	return open, close
end

local function Texture_OnEnter(self)
	if self:IsEnabled() then
		if self.backdrop then
			self.backdrop:SetBackdropColor(0.1, 0.1, 0.1, 1)
		else
			self.__texture:SetVertexColor(0, 0.6, 1)
		end
	end
end

local function Texture_OnLeave(self)
	if self.backdrop then
		self.backdrop:SetBackdropColor(0, 0, 0, 0.3)
	else
		self.__texture:SetVertexColor(1, 1, 1)
	end
end

local function updateCollapseTexture(texture, collapsed)
	if collapsed then
		texture:SetTexCoord(0, .4375, 0, .4375)
	else
		texture:SetTexCoord(.5625, 1, 0, .4375)
	end
end

function T.ReskinCollapse(self)
	self:SetNormalTexture("")
	self:SetHighlightTexture("")
	self:SetPushedTexture("")
	self:SetDisabledTexture("")

	self:CreateBackdrop("ClassColor")
	self.backdrop:ClearAllPoints()
	self.backdrop:SetSize(15, 15)
	self.backdrop:SetPoint("TOPLEFT", self:GetNormalTexture())
	self.backdrop:SetBackdropColor(0, 0, 0, 0.3)

	self.__texture = self.backdrop:CreateTexture(nil, "OVERLAY")
	self.__texture:SetPoint("CENTER")
	self.__texture:SetSize(8, 8)
	self.__texture:SetTexture("Interface\\Buttons\\UI-PlusMinus-Buttons")
	self.__texture.DoCollapse = updateCollapseTexture

	self:HookScript("OnEnter", Texture_OnEnter)
	self:HookScript("OnLeave", Texture_OnLeave)
end
----------------------------------------------------------------------------------------
-- Events function
----------------------------------------------------------------------------------------
local events = {}

local EventFrame = CreateFrame("Frame")
EventFrame:SetScript("OnEvent", function(_, event, ...)
	for func in pairs(events[event]) do
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			func(event, CombatLogGetCurrentEventInfo())
		else
			func(event, ...)
		end
	end
end)

T.RegisterEvent = function(event, func, unit1, unit2)
	if not events[event] then
		events[event] = {}
		if unit1 then
			EventFrame:RegisterUnitEvent(event, unit1, unit2)
		else
			EventFrame:RegisterEvent(event)
		end
	end

	events[event][func] = true
end

T.UnregisterEvent = function(event, func)
	local funcs = events[event]
	if funcs and funcs[func] then
		funcs[func] = nil

		if not next(funcs) then
			events[event] = nil
			EventFrame:UnregisterEvent(event)
		end
	end
end

----------------------------------------------------------------------------------------
--	Reskin tdGUI and L_DropDownList
----------------------------------------------------------------------------------------
local Skin = CreateFrame("Frame")
Skin:RegisterEvent("ADDON_LOADED")
Skin:SetScript("OnEvent", function()
	-- tdGUI
	local function ReskinTooltip(self, a)
		if not self then return end
		if self:IsForbidden() then return end

		if not self.tipStyled then
			self:StripTextures()
			self:DisableDrawLayer("BACKGROUND")
			self:CreateBackdrop("Transparent")

			self.tipStyled = true
		end
	end

	local GUI = _G.LibStub and _G.LibStub("tdGUI-1.0", true)
	if not GUI then return end

	local DropMenu = GUI:GetClass("DropMenu")
	if DropMenu then
		hooksecurefunc(DropMenu, "Open", function(self, level, ...)
			level = level or 1
			local menu = self.menuList[level]
			if menu then
				if not menu.styled then
					ReskinTooltip(menu)

					local scrollBar = menu.scrollBar or menu.ScrollBar
					if scrollBar then
						T.SkinScrollBar(scrollBar)
					end

					menu.styled = true
				end
			end
		end)

		hooksecurefunc(DropMenu, "UpdateItems", function(self)
			for i = 1, #self._buttons do
				local bu = self:GetButton(i)
				local left, right = self:GetPadding()
				if bu:IsShown() and left and right then
					local hl = bu:GetHighlightTexture()

					if not bu.styled then
						hl:SetColorTexture(1, 1, 1, 0.3)
						bu.styled = true
					end

					hl:SetPoint("TOPLEFT", -left + T.mult, 0)
					hl:SetPoint("BOTTOMRIGHT", right - self:GetScrollBarFixedWidth()- T.mult, 0)
				end
			end
		end)
	end

	-- L_DropDownList
	local function reskinDropDownMenu(level, isQuestie)
		if not level then level = 1 end

		local listFrameName = isQuestie and "L_DropDownListQuestie"..level or "L_DropDownList"..level
		local listFrame = _G[listFrameName]

		if not listFrame.backdrop then
			listFrame:CreateBackdrop("Transparent")
		end

		local backdrop = listFrame.Backdrop
		if backdrop then backdrop:Hide() end
		backdrop = listFrame.MenuBackdrop
		if backdrop then backdrop:Hide() end
		backdrop = listFrame.Border
		if backdrop then backdrop:Hide() end
		backdrop = _G[listFrameNameBackdrop]
		if backdrop then backdrop:Hide() end
		backdrop = _G[listFrameNameMenuBackdrop]
		if backdrop then backdrop:Hide() end
		backdrop =  _G[listFrameNameBorder]
		if backdrop then backdrop:Hide() end

		local maxButtons = isQuestie and _G.L_UIDROPDOWNMENUQUESTIE_MAXBUTTONS or _G.L_UIDROPDOWNMENU_MAXBUTTONS
		for i = 1, maxButtons do
			local bu = _G[listFrameName.."Button"..i]
			local _, _, _, x = bu:GetPoint()
			if bu:IsShown() and x then
				local check = _G[listFrameName.."Button"..i.."Check"]
				local uncheck = _G[listFrameName.."Button"..i.."UnCheck"]
				local hl = _G[listFrameName.."Button"..i.."Highlight"]
				local arrow = _G[listFrameName.."Button"..i.."ExpandArrow"]

				if not bu.backdrop then
					bu:CreateBackdrop("Transparent")
					bu.backdrop:ClearAllPoints()
					bu.backdrop:SetPoint("CENTER", check)
					bu.backdrop:SetSize(12, 12)
					hl:SetColorTexture(1, 1, 1, 0.3)
				end

				bu.backdrop:Hide()
				hl:SetPoint("TOPLEFT", -x + T.mult, 0)
				hl:SetPoint("BOTTOMRIGHT", listFrame:GetWidth() - bu:GetWidth() - x - T.mult, 0)
				if uncheck then uncheck:SetTexture("") end

				if not bu.notCheckable then
					local _, co = check:GetTexCoord()
					if co == 0 then
						check:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
						check:SetVertexColor(1, 0.82, 0, 1)
						check:SetSize(20, 20)
						check:SetDesaturated(true)
					else
						check:SetColorTexture(1, 0.82, 0, 0.6)
						check:SetSize(8, 8)
						check:SetDesaturated(false)
					end

					check:SetTexCoord(0, 1, 0, 1)
					bu.backdrop:Show()
				end
			end
		end
	end

	local LibDropMenu = LibStub("LibUIDropDownMenu-4.0", true)
	if LibDropMenu then
		hooksecurefunc(LibDropMenu, "ToggleDropDownMenu", function(self, level, ...)
			reskinDropDownMenu(level)
		end)
	end

	local LibDropMenuQuestie = LibStub("LibUIDropDownMenuQuestie-4.0", true)
	if LibDropMenuQuestie then
		hooksecurefunc(LibDropMenuQuestie, "ToggleDropDownMenu", function(self, level, ...)
			reskinDropDownMenu(level, true)
		end)
	end

	-- LibUIDropDownMenu-2.0
	if _G.L_ToggleDropDownMenu then
		hooksecurefunc(_G, "L_ToggleDropDownMenu", function(level, ...)
			reskinDropDownMenu(level)
		end)
	end
end)